@@ -5,17 +5,22 @@ length and estimated number of records if sniffing a URL, file size, number of f
 field names & data types) using a Viterbi algorithm.
 (https://en.wikipedia.org/wiki/Viterbi_algorithm)
 
-NOTE: This command "sniffs" a CSV's schema by sampling the first n rows of a file.
-Its inferences are sometimes wrong if the sample is not large enough (use --sample 
-to adjust). 
+On Linux, `sniff` also acts as a general file type detector using the libmagic library
+and returns the detected mime type, file size and last modified date if the file is not
+a CSV. If --no-infer is enabled, it doesn't even bother to infer the CSV's schema.
+
+On macOS and Windows however, `sniff` is only a CSV dialect detector and does not
+detect other file types. It can only sniff files with the "csv", "tsv", "tab" and
+"txt" extensions, and snappy compressed variants of these formats (e.g. "csv.sz",
+"tsv.sz", etc.) extensions.
+
+NOTE: This command "sniffs" a CSV's schema by sampling the first n rows (default: 1000)
+of a file. Its inferences are sometimes wrong if the sample is not large enough 
+(use --sample to adjust) or the file is too small to infer a pattern. 
 
 If you want more robust, guaranteed schemata, use the "schema" or "stats" commands
 instead as they scan the entire file.
 
-On Linux, `sniff` also acts as a general file type detector and returns the detected
-mime type if the file is not a CSV. It uses the "libmagic" library to do this.
-See https://github.com/robo9k/rust-magic for more info.
-
 For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_sniff.rs.
 
 Usage:
@@ -27,7 +32,7 @@ sniff arguments:
                              or a URL (http and https schemes supported).
 
                              Note that when input is a URL, sniff will automatically
-                             download the file to a temporary file and sniff it. It
+                             download the sample to a temporary file and sniff it. It
                              will create the file using csv::QuoteStyle::NonNumeric
                              so the sniffed schema may not be the same as the original.
                              This is done to increase the chances of sniffing the
@@ -41,21 +46,24 @@ sniff options:
                              When it is zero, the entire file will be sampled.
                              When the input is a URL, the sample size dictates
                              how many lines to sample without having to
-                             download the entire file.
+                             download the entire file. Ignored when --no-infer is enabled.
                              [default: 1000]
-    --prefer-dmy             Prefer to parse dates in dmy format.
-                             Otherwise, use mdy format.
+    --prefer-dmy             Prefer to parse dates in dmy format. Otherwise, use mdy format.
+                             Ignored when --no-infer is enabled.
     --json                   Return results in JSON format.
     --pretty-json            Return results in pretty JSON format.
     --save-urlsample <file>  Save the URL sample to a file.
                              Valid only when input is a URL.
-    --timeout <secs>         Timeout for URL requests in seconds.
+    --timeout <secs>         Timeout when sniffing URLs in seconds.
                              [default: 30]
     --user-agent <agent>     Specify a custom user agent to use when sniffing a CSV on a URL.
                              Try to follow the syntax here -
                              https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/User-Agent
     --stats-types            Use the same data type names as `stats`.
                              (Unsigned, Signed => Integer, Text => String, everything else the same)
+    --no-infer               Do not infer the schema. Only return the file's mime type, size and
+                             last modified date. Use this to use sniff as a general mime type detector.
+                             Valid only on Linux.
 
 Common options:
     -h, --help               Display this message
@@ -106,6 +114,8 @@ struct Args {
     flag_timeout:        u16,
     flag_user_agent:     Option<String>,
     flag_stats_types:    bool,
+    #[cfg(all(target_os = "linux", feature = "magic"))]
+    flag_no_infer:       bool,
 }
 
 #[derive(Serialize, Deserialize, Default, Debug)]
@@ -302,7 +312,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     ))
                     .send()
                     .await
-                    .or(Err(format!("Failed to GET from '{url}'")))?;
+                    .map_err(|e| format!("Download failed: {e}"))?;
 
                 let last_modified = match res.headers().get("Last-Modified") {
                     Some(lm) => match lm.to_str() {
@@ -386,24 +396,33 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
                 // on linux, we can short-circuit downloading by
                 // checking the file type from the first chunk
+                // the unused_muts are here to suppress warnings
+                // when the magic feature is not enabled
                 #[allow(unused_mut)]
                 let mut shortcircuit_flag = false;
                 #[allow(unused_mut)]
                 let mut firstchunk = Bytes::new();
+                #[allow(unused_mut)]
+                let mut magic_flag = false;
 
                 // download chunks until we have the desired sample size
                 while let Some(item) = stream.next().await {
-                    chunk = item.or(Err("Error while downloading file".to_string()))?;
+                    chunk = item.map_err(|_| "Error while downloading file")?;
 
                     file.write_all(&chunk)
-                        .map_err(|_| "Error while writing to file".to_string())?;
+                        .map_err(|_| "Error while writing to file")?;
                     let chunk_len = chunk.len();
 
+                    #[cfg(all(target_os = "linux", feature = "magic"))]
+                    {
+                        // we're using magic!
+                        magic_flag = true;
+                    }
+
                     // on linux, we can short-circuit downloading
                     // by checking the file type from the first chunk
                     #[cfg(all(target_os = "linux", feature = "magic"))]
-                    if downloaded == 0 && !snappy_flag && chunk_len >= util::FIRST_CHUNK_BUFFER_SIZE
-                    {
+                    if downloaded == 0 && !snappy_flag {
                         let mime = util::sniff_filetype_from_buffer(&chunk)?;
                         if !mime.starts_with("text/") && mime != "application/csv" {
                             shortcircuit_flag = true;
@@ -453,9 +472,8 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
                 // keep the temporary file around so we can sniff it later
                 // we'll delete it when we're done
-                let (mut tmp_file, path) = wtr_file
-                    .keep()
-                    .or(Err("Cannot keep temporary file".to_string()))?;
+                let (mut tmp_file, path) =
+                    wtr_file.keep().map_err(|_| "Cannot keep temporary file")?;
 
                 let wtr_file_path;
                 let mut downloaded_records = 0_usize;
@@ -519,7 +537,11 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     file_to_sniff: wtr_file_path,
                     tempfile_flag: true,
                     retrieved_size: downloaded,
-                    file_size: if total_size == usize::MAX {
+                    file_size: if magic_flag && total_size == usize::MAX {
+                        // we're using magic and the server didn't give us content length
+                        // so send usize::MAX - 1 to indicate that we don't know the file size
+                        usize::MAX - 1
+                    } else if total_size == usize::MAX {
                         // the server didn't give us content length, so we just
                         // downloaded the entire file. downloaded variable
                         // is the total size of the file
@@ -620,7 +642,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
         drop(stdin_handle);
         let (file, path) = stdin_file
             .keep()
-            .or(Err("Cannot keep temporary file".to_string()))?;
+            .map_err(|_| "Cannot keep temporary file")?;
 
         if !util::isutf8_file(&path)? {
             return fail_clierror!("stdin input is not UTF8-encoded");
@@ -628,7 +650,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
         let metadata = file
             .metadata()
-            .map_err(|_| "Cannot get metadata for stdin file".to_string())?;
+            .map_err(|_| "Cannot get metadata for stdin file")?;
 
         let file_size = metadata.len() as usize;
         // set last_modified to now in RFC3339 format
@@ -692,24 +714,66 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
     #[cfg(all(target_os = "linux", feature = "magic"))]
     // we also accept text/* files, as sniff may still be able to suss out
     // if the file is a CSV, even if its not using a CSV extension
-    if file_type != "application/csv" && !file_type.starts_with("text/") {
+    // we can also be here if the user has specified --no-infer
+    if (file_type != "application/csv" && !file_type.starts_with("text/")) || args.flag_no_infer {
         cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
+
+        let size = if sfile_info.file_size >= usize::MAX - 1 {
+            "Unknown".to_string()
+        } else {
+            sfile_info.file_size.to_string()
+        };
+        let last_modified = sfile_info.last_modified.clone();
+
         if args.flag_json || args.flag_pretty_json {
-            let json_result = json!({
-                "errors": [{
-                    "title": "sniff error",
+            if args.flag_no_infer {
+                let json_result = json!({
+                    "title": "sniff mime type",
                     "detail": format!("File is not a CSV file. Detected mime type: {file_type}"),
                     "meta": {
-                        "detected_mime_type": file_type
+                        "detected_mime_type": file_type,
+                        "size": size,
+                        "last_modified": last_modified,
                     }
-                }]
-            });
-            if args.flag_pretty_json {
-                return fail_clierror!("{}", serde_json::to_string_pretty(&json_result).unwrap());
+                });
+                if args.flag_pretty_json {
+                    woutinfo!("{}", serde_json::to_string_pretty(&json_result).unwrap());
+                    return Ok(());
+                }
+                woutinfo!("{json_result}");
+                return Ok(());
+            } else {
+                let json_result = json!({
+                    "errors": [{
+                        "title": "sniff error",
+                        "detail": format!("File is not a CSV file. Detected mime type: {file_type}"),
+                        "meta": {
+                            "detected_mime_type": file_type,
+                            "size": size,
+                            "last_modified": last_modified,
+                        }
+                    }]
+                });
+                if args.flag_pretty_json {
+                    return fail_clierror!(
+                        "{}",
+                        serde_json::to_string_pretty(&json_result).unwrap()
+                    );
+                }
+                return fail_clierror!("{json_result}");
             }
-            return fail_clierror!("{json_result}");
         }
-        return fail_clierror!("File is not a CSV file. Detected mime type: {file_type}");
+        if args.flag_no_infer {
+            woutinfo!(
+                "Detected mime type: {file_type}, size: {size}, last modified: {last_modified}"
+            );
+            return Ok(());
+        } else {
+            return fail_clierror!(
+                "File is not a CSV file. Detected mime type: {file_type}, size: {size}, last \
+                 modified: {last_modified}"
+            );
+        }
     }
 
     let conf = Config::new(&Some(sfile_info.file_to_sniff.clone()))
