@@ -597,9 +597,7 @@ fn excel_metadata_pretty_json() {
     cmd.arg("--metadata").arg("J").arg(xls_file);
 
     let got: String = wrk.stdout(&mut cmd);
-    let expected: &str = r#"{
-  "filename": "excel-xls.xls",
-  "format": "xls",
+    let expected: &str = r#""format": "xls",
   "num_sheets": 8,
   "sheet": [
     {
@@ -756,7 +754,7 @@ fn excel_metadata_pretty_json() {
     }
   ]
 }"#;
-    assert_eq!(got, expected);
+    assert!(got.ends_with(expected));
     wrk.assert_success(&mut cmd);
 }
 
@@ -994,3 +992,22 @@ fn excel_large_floats() {
     assert_eq!(got, expected);
     wrk.assert_success(&mut cmd);
 }
+
+#[test]
+fn excel_range_empty_sheet() {
+    let wrk = Workdir::new("excel_range_empty_sheet");
+
+    let xls_file = wrk.load_test_file("excel-range.xlsx");
+
+    let mut cmd = wrk.command("excel");
+    cmd.arg(xls_file);
+    cmd.arg("--range").arg("a2:b");
+    cmd.arg("-s").arg("Sheet2");
+
+    assert!(wrk
+        .output_stderr(&mut cmd)
+        .matches("larger than sheet")
+        .min()
+        .is_some());
+    wrk.assert_err(&mut cmd);
+}
